@@ -134,12 +134,25 @@ impl fmt::Display for EncryptError {
     }
 }
 
+#[derive(Debug)]
+pub(crate) struct DetectedPowerShellCorruptionError;
+
+impl fmt::Display for DetectedPowerShellCorruptionError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        wlnfl!(f, "err-detected-powershell-corruption")?;
+        wfl!(f, "rec-detected-powershell-corruption")
+    }
+}
+
+impl std::error::Error for DetectedPowerShellCorruptionError {}
+
 pub(crate) enum DecryptError {
     Age(age::DecryptError),
     ArmorFlag,
     IdentityRead(age::cli_common::ReadError),
     Io(io::Error),
     MissingIdentities,
+    MixedIdentityAndPassphrase,
     MixedIdentityAndPluginName,
     PassphraseFlag,
     PassphraseTimedOut,
@@ -195,6 +208,9 @@ impl fmt::Display for DecryptError {
                 wlnfl!(f, "err-dec-missing-identities")?;
                 wlnfl!(f, "rec-dec-missing-identities")
             }
+            DecryptError::MixedIdentityAndPassphrase => {
+                wfl!(f, "err-dec-mixed-identity-passphrase")
+            }
             DecryptError::MixedIdentityAndPluginName => {
                 wfl!(f, "err-mixed-identity-and-plugin-name")
             }
